package ru.dsi.bgbilling.modules.inet.accounting.detail;

import org.apache.log4j.Logger;
import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.application.server.Lifecycle;
import ru.bitel.bgbilling.kernel.event.EventListener;
import ru.bitel.bgbilling.kernel.event.EventListenerContext;
import ru.bitel.bgbilling.kernel.event.EventProcessor;
import ru.bitel.bgbilling.kernel.event.EventWorker;
import ru.bitel.bgbilling.modules.inet.accounting.Accounting;
import ru.bitel.bgbilling.modules.inet.api.server.bean.InetDeviceMap;
import ru.bitel.bgbilling.modules.inet.api.server.event.InetReloadEvent;
import ru.bitel.bgbilling.server.util.Setup;
import ru.bitel.bgbilling.server.util.SetupParam;
import ru.dsi.bgbilling.modules.inet.accounting.detail.event.InetClientDetailCreateEvent;

import java.beans.ConstructorProperties;
import java.util.HashSet;
import java.util.Set;

/**
 * Воркер для обработки заданий создания файлов детализации из Web-интерфейса клиента
 * Работает параллельно стандартному
 *
 * Конфигурация в inet-accounting.xml:
 * <application context="accounting">
 *     ...
 *     <param name="detail.client.dir" value="/usr/local/ipdetail" />
 *     ...
 *
 *     <context name="collector">
 *         <context name="detail">
 *             <!-- Cоздание обработчика flow детализации -->
 *             <bean name="detailWorker" class="ru.bitel.bgbilling.modules.inet.accounting.detail.InetDetailWorker"/>
 *             <!-- Собственный обработчик flow детализации из ЛК-->
 *             <bean name="clientDetailWorker" class="ru.dsi.bgbilling.modules.inet.accounting.detail.InetClientDetailWorker"/>
 *         </context>
 *     </context>
 *
 * </application>
 */
public class InetClientDetailWorker extends EventWorker
        implements Lifecycle
{
    private static final Logger logger = Logger.getLogger(InetClientDetailWorker.class);
    private final Setup setup;
    private final Accounting accounting;

    @ConstructorProperties({"setup", "accounting"})
    public InetClientDetailWorker(Setup setup, Accounting accounting)
            throws BGException
    {
        super(SetupParam.getModuleId(setup));

        this.setup = setup;
        this.accounting = accounting;

        this.context = new EventListenerContext(setup, this.moduleId, SetupParam.getApplicationName());

        EventProcessor.getInstance().addListener(new EventListener<InetReloadEvent>()
        {
            public void notify(InetReloadEvent e, EventListenerContext ctx)
                    throws BGException
            {
                new Thread()
                {
                    public void run()
                    {
                        try
                        {
                            InetClientDetailWorker.this.setConsumer(
                                    InetClientDetailCreateEvent.class,
                                    InetClientDetailWorker.getQuery(
                                            InetClientDetailWorker.this.setup,
                                            InetClientDetailWorker.this.accounting)
                            );
                        }
                        catch (BGException e)
                        {
                            InetClientDetailWorker.logger.error(e.getMessage(), e);
                        }
                    }
                }
                        .start();
            }
        }
                , InetReloadEvent.class, this.moduleId, null);
    }

    private static String getQuery(Setup setup, Accounting accounting)
            throws BGException
    {
        Set<Integer> childrenDeviceIds;
        if ((accounting != null) && (accounting.childrenDeviceIds != null))
        {
            childrenDeviceIds = accounting.childrenDeviceIds;
        }
        else
        {
            int rootDeviceId = setup.getInt("rootDeviceId", 0);

            InetDeviceMap deviceMap = InetDeviceMap.getInstance(SetupParam.getModuleId(setup));
            childrenDeviceIds = deviceMap.get(rootDeviceId).getDescendantIds();
            childrenDeviceIds = new HashSet<Integer>(childrenDeviceIds);
            childrenDeviceIds.add(rootDeviceId);
        }

        StringBuilder sb = new StringBuilder(20).append('(');
        for (Integer id : childrenDeviceIds)
        {
            sb.append("deviceId=").append(id).append(" OR ");
        }
        sb.setLength(sb.length() - 4);
        sb.append(')');

        return sb.toString();
    }

    public void start()
            throws Exception
    {
        setConsumer(InetClientDetailCreateEvent.class, getQuery(this.setup, this.accounting));

        new Thread(this, "client-detail-worker").start();
    }

    public void stop()
            throws Exception
    {
        shutdown();
    }

    protected boolean runWorker()
            throws BGException
    {
        return doTasks();
    }

    public Object doTask(Object task)
            throws BGException
    {
        try
        {
            InetClientDetailCreateEvent e = (InetClientDetailCreateEvent)task;

            new InetFlowClientDetailMaker().createDetail(this.context.getSetup(), this.context.getConnectionSet(), e);
        }
        catch (Exception e)
        {
            throw new BGException(e);
        }

        return null;
    }
}